package com.example

import at.favre.lib.crypto.bcrypt.BCrypt
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.transactions.transaction
import java.net.InetAddress
import java.util.Random

object AuthService {

    //USERS
    fun registerUser(name: String, email: String, password: String): Boolean {
        if (!isEmailValid(email)) {
            return false
        }

        val hashedPassword = hashPassword(password)
        return transaction {
            if (Users.select { Users.email eq email }.empty()) {
                Users.insert {
                    it[Users.name] = name
                    it[Users.email] = email
                    it[Users.passwordHash] = hashedPassword
                }
                true
            } else {
                false
            }
        }
    }

    fun loginUser(email: String, password: String): Boolean {
        val user = transaction {
            Users.select { Users.email eq email }
                .mapNotNull { it[Users.passwordHash] }
                .firstOrNull()
        }
        return user != null && checkPassword(password, user)
    }

    fun generateResetCode(email: String): String? {
        val code = generate4DigitCode()
        return transaction {
            val updatedRows = Users.update({ Users.email eq email }) {
                it[resetToken] = code
            }
            if (updatedRows > 0) code else null
        }
    }

    fun resetPassword(code: String, newPassword: String): Boolean {
        return transaction {
            val email = Users.select { Users.resetToken eq code }
                .mapNotNull { it[Users.email] }
                .firstOrNull()

            if (email != null) {
                val hashedPassword = hashPassword(newPassword)
                Users.update({ Users.email eq email }) {
                    it[passwordHash] = hashedPassword
                    it[resetToken] = null
                }
                true
            } else {
                false
            }
        }
    }

    private fun isEmailValid(email: String): Boolean {
        return try {
            val domain = email.substringAfter("@")
            val addresses = InetAddress.getAllByName(domain)
            addresses.isNotEmpty()
        } catch (e: Exception) {
            false
        }
    }

    private fun generate4DigitCode(): String {
        return (1000..9999).random().toString()
    }

    private fun hashPassword(password: String): String {
        return BCrypt.withDefaults().hashToString(12, password.toCharArray())
    }

    private fun checkPassword(password: String, hash: String): Boolean {
        return BCrypt.verifyer().verify(password.toCharArray(), hash).verified
    }


    //SNEAKERS
    fun selectSneakers(): List<Sneaker> {
        try {
            return transaction {
                Sneakers.selectAll().map {
                    Sneaker(
                        id = it[Sneakers.id],
                        name = it[Sneakers.name],
                        description = it[Sneakers.description],
                        cost = it[Sneakers.cost],
                        discount = it[Sneakers.discount],
                        photo = it[Sneakers.photo],
                        gender = it[Sneakers.gender],
                        bootSize = it[Sneakers.bootSize]
                    )
                }
            }
        } catch (e: Exception) {
            throw RuntimeException("Error selecting sneakers: ${e.message}", e)
        }
    }
}


